// @flow strict

/* ::
type Config = {
  env?: { [key: string]: boolean, ... },
  parser?: string,
  parserOptions?: { [key: string]: mixed,... },
  extends?: string[],
  plugins?: string[],
  rules?: { [rule: string]: string | mixed[], ... },
  settings?: { [option: string]: mixed, ... },
  overrides?: Array<{ files: Array<string>,  ...Config, ... }>,
};
*/

const presets = [
  'airbnb',
  'airbnb/hooks',
  'plugin:import/errors',
  'plugin:import/warnings',
  'plugin:import/typescript',
  'plugin:@next/next/recommended',
  'plugin:react/recommended',
  'plugin:testing-library/react',
  'plugin:sonarjs/recommended',
  'plugin:unicorn/recommended',
  'prettier',
  'prettier/react',
  'prettier/unicorn',
];

const plugins = [
  '@emotion',
  'flowtype',
  // 'graphql',
  'import',
  'jsx-a11y',
  '@next/next',
  'react',
  'react-hooks',
  'sonarjs',
  'testing-library',
  'unicorn',
];

const rules = {
  camelcase: 'off',
  'no-else-return': 'off',
  'no-console': 'warn',
  'no-debugger': 'warn',
  'no-multiple-empty-lines': ['error', { max: 3, maxEOF: 0 }],
  'no-underscore-dangle': 'off',
  'no-unused-expressions': [
    'warn',
    { allowShortCircuit: true, allowTernary: true, allowTaggedTemplates: true },
  ],
  'no-unused-vars': 'warn',
  'no-useless-computed-key': 'warn',
  'no-useless-constructor': 'warn',
  'no-useless-rename': [
    'warn',
    { ignoreDestructuring: false, ignoreImport: false, ignoreExport: false },
  ],
  'padded-blocks': ['error', { blocks: 'never', switches: 'never', classes: 'never' }],
  'prefer-const': 'warn',
  'spaced-comment': ['error', 'always', { markers: ['/'] }],

  '@emotion/jsx-import': 'error',
  '@emotion/pkg-renaming': 'error',
  '@emotion/styled-import': 'error',
  '@emotion/syntax-preference': ['warn', 'object'],

  // 'graphql/template-strings': ['error', { env: 'apollo', validators: 'all' }],
  // 'graphql/template-strings': ['error', { env: 'relay', validators: 'all' }],

  'import/default': 'warn',
  'import/named': 'warn',
  'import/namespace': 'error',
  'import/extensions': ['warn', 'never'],

  'jsx-a11y/anchor-is-valid': 'off', // next.js
  'jsx-a11y/aria-props': 'error',
  'jsx-a11y/aria-role': 'error',
  'jsx-a11y/heading-has-content': 'error',
  'jsx-a11y/href-no-hash': 'off',
  'jsx-a11y/label-has-for': 'off',
  'jsx-a11y/mouse-events-have-key-events': 'error',
  'jsx-a11y/role-has-required-aria-props': 'error',
  'jsx-a11y/role-supports-aria-props': 'error',
  'jsx-a11y/img-redundant-alt': 'error',
  'jsx-a11y/no-access-key': 'error',

  'react/jsx-indent': 'off',
  'react/require-default-props': 'off',
  'react/forbid-prop-types': ['error', { forbid: ['any'] }],
  'react/default-props-match-prop-types': 'off', // flow/ts
  'react/jsx-curly-brace-presence': 'off', // styled-jsx
  'react/jsx-filename-extension': ['error', { extensions: ['.js', '.mjs', '.tsx'] }],
  'react/no-array-index-key': 'off',
  'react/no-unused-prop-types': 'off', // flow/ts
  'react/jsx-indent-props': 'off',
  'react/jsx-wrap-multilinis': 'off',
  'react/sort-comp': 'off',
  'react/jsx-props-no-spreading': 'off', // todo

  'react-hooks/rules-of-hooks': 'error',
  'react-hooks/exhaustive-deps': 'warn',

  'sonarjs/no-duplicate-string': ['error', 5],

  'unicorn/filename-case': 'off',
  'unicorn/catch-error-name': ['warn', { name: 'err' }],
  'unicorn/prevent-abbreviations': 'off',
};

const settings = {
  'import/extensions': ['.js', '.mjs', '.ts', '.jsx', '.tsx'],
};

const jestOverride = {
  files: [
    '**/*.test.js',
    '**/*.test.ts',
    '**/*.test.tsx',
    '**/*.spec.js',
    '**/*.spec.ts',
    '**/*.spec.tsx',
  ],
  env: { jest: true },
};

const typescriptOverride = {
  files: ['**/*.ts', '**/*.tsx'],
  parser: '@typescript-eslint/parser',
  extends: [...presets, 'plugin:import/typescript'],
  parserOptions: {
    ecmaVersion: 2018,
    sourceType: 'module',
    ecmaFeatures: { jsx: true },
    warnOnUnsupportedTypeScriptVersion: false,
  },
  plugins: plugins
    .filter((plugin) => plugin !== 'flowtype')
    .concat(['@typescript-eslint']),

  rules: {
    ...rules,
    'no-unused-vars': 'warn',
    'flowtype/define-flow-type': 'error',
    'flowtype/require-valid-file-annotation': 'error',
    'flowtype/use-flow-type': 'error',

    // (see https://github.com/microsoft/TypeScript/issues/14306)
    // prettier-ignore
    'no-restricted-globals': [ 'error', 'postMessage', 'blur', 'focus', 'close', 'frames', 'self', 'parent', 'opener', 'top', 'length', 'closed', 'location', 'origin', 'name', 'locationbar', 'menubar', 'personalbar', 'scrollbars', 'statusbar', 'toolbar', 'status', 'frameElement', 'navigator', 'customElements', 'external', 'screen', 'innerWidth', 'innerHeight', 'scrollX', 'pageXOffset', 'scrollY', 'pageYOffset', 'screenX', 'screenY', 'outerWidth', 'outerHeight', 'devicePixelRatio', 'clientInformation', 'screenLeft', 'screenTop', 'defaultStatus', 'defaultstatus', 'styleMedia', 'onanimationend', 'onanimationiteration', 'onanimationstart', 'onsearch', 'ontransitionend', 'onwebkitanimationend', 'onwebkitanimationiteration', 'onwebkitanimationstart', 'onwebkittransitionend', 'isSecureContext', 'onabort', 'onblur', 'oncancel', 'oncanplay', 'oncanplaythrough', 'onchange', 'onclick', 'onclose', 'oncontextmenu', 'oncuechange', 'ondblclick', 'ondrag', 'ondragend', 'ondragenter', 'ondragleave', 'ondragover', 'ondragstart', 'ondrop', 'ondurationchange', 'onemptied', 'onended', 'onerror', 'onfocus', 'oninput', 'oninvalid', 'onkeydown', 'onkeypress', 'onkeyup', 'onload', 'onloadeddata', 'onloadedmetadata', 'onloadstart', 'onmousedown', 'onmouseenter', 'onmouseleave', 'onmousemove', 'onmouseout', 'onmouseover', 'onmouseup', 'onmousewheel', 'onpause', 'onplay', 'onplaying', 'onprogress', 'onratechange', 'onreset', 'onresize', 'onscroll', 'onseeked', 'onseeking', 'onselect', 'onstalled', 'onsubmit', 'onsuspend', 'ontimeupdate', 'ontoggle', 'onvolumechange', 'onwaiting', 'onwheel', 'onauxclick', 'ongotpointercapture', 'onlostpointercapture', 'onpointerdown', 'onpointermove', 'onpointerup', 'onpointercancel', 'onpointerover', 'onpointerout', 'onpointerenter', 'onpointerleave', 'onafterprint', 'onbeforeprint', 'onbeforeunload', 'onhashchange', 'onlanguagechange', 'onmessage', 'onmessageerror', 'onoffline', 'ononline', 'onpagehide', 'onpageshow', 'onpopstate', 'onrejectionhandled', 'onstorage', 'onunhandledrejection', 'onunload', 'performance', 'stop', 'open', 'print', 'captureEvents', 'releaseEvents', 'getComputedStyle', 'matchMedia', 'moveTo', 'moveBy', 'resizeTo', 'resizeBy', 'getSelection', 'find', 'createImageBitmap', 'scroll', 'scrollTo', 'scrollBy', 'onappinstalled', 'onbeforeinstallprompt', 'crypto', 'ondevicemotion', 'ondeviceorientation', 'ondeviceorientationabsolute', 'indexedDB', 'webkitStorageInfo', 'chrome', 'visualViewport', 'speechSynthesis', 'webkitRequestFileSystem', 'webkitResolveLocalFileSystemURL', 'openDatabase']
  },
};

const config /* : Config*/ = {
  extends: presets,
  plugins,
  rules,
  settings,
  overrides: [jestOverride, typescriptOverride],
};

module.exports = config;
